@@ -42,10 +42,11 @@
 #include "errorchain.h"
 #include "ldif.h"
 #include "formfill.h"
+#include "encode.h"
 #include "input.h"
 #include "debug.h"
 
-#include "icons/warning.xpm"
+#include "../icons/warning.xpm"
 
 extern GtkWidget *statusbar, *mainwin;
 extern struct gq_config config;
@@ -207,6 +208,7 @@ gboolean gnode_to_tree_callback(GHashTable *hash, GNode *node, gpointer ctree_no
      int node_added_sequence;
      char *labels[2] = { 0 };
      char **exploded_dn = 0;
+     char *decoded_label;
 
      entry = (struct node_entry *) node->data;
      ctreeroot = g_hash_table_lookup(hash, "ctreeroot");
@@ -218,6 +220,10 @@ gboolean gnode_to_tree_callback(GHashTable *hash, GNode *node, gpointer ctree_no
 	  exploded_dn = ldap_explode_dn(labels[0], FALSE);
 	  labels[0] = exploded_dn[0];
      }
+
+     decoded_label = decoded_string(labels[0]);
+     strcpy(labels[0], decoded_label);
+     g_free(decoded_label);
   
      new_item = gtk_ctree_insert_node(GTK_CTREE(ctreeroot), GTK_CTREE_NODE(ctree_node),
 				      0, labels, 0, 0, 0, 0, 0, 0, 0);
@@ -1269,7 +1275,7 @@ void dump_subtree(GtkWidget *widget, GHashTable *hash)
 	  statusbar_msg(gmessage->str);
 	  msg = ldap_search_s(ld, (char *) bases->data, LDAP_SCOPE_SUBTREE,
 			      "(objectclass=*)", NULL, 0, &res);
-	  if(msg != -1)
+	  if(msg == LDAP_SUCCESS)
 	       for(e = ldap_first_entry(ld, res); e; e = ldap_next_entry(ld, e)) {
 		    ldif_entry_out(out, ld, e);
 		    num_entries++;
