@@ -37,6 +37,7 @@
 #include "common.h"
 #include "util.h"
 #include "template.h"
+#include "filter.h"
 #include "errorchain.h"
 #include "debug.h"
 
@@ -76,6 +77,11 @@ struct keywordlist configwords[] = {
      { "template",        T_TEMPLATE,          NEEDS_CLOSE },
      { "objectclass",     T_OBJECTCLASS,       NEEDS_CLOSE|NEEDS_DATA },
 
+     /* filters */
+     { "filter",          T_FILTER,            NEEDS_CLOSE },
+     { "ldapfilter",      T_LDAPFILTER,        NEEDS_CLOSE|NEEDS_DATA },
+     { "servername",      T_SERVERNAME,        NEEDS_CLOSE|NEEDS_DATA },
+
      { "", 0 }
 };
 
@@ -137,6 +143,21 @@ struct gq_template *new_template(void)
 }
 
 
+/* malloc and init gq_template struct */
+struct gq_filter *new_filter(void)
+{
+     struct gq_filter *filter;
+
+     filter = MALLOC(sizeof(struct gq_filter), "struct gq_filter");
+     filter->name[0] = '\0';
+     filter->ldapfilter[0] = '\0';
+     filter->servername[0] = '\0';
+     filter->basedn[0] = '\0';
+
+     return(filter);
+}
+
+
 /* bloody hell */
 void delete_ldapserver(struct ldapserver *dserver)
 {
@@ -186,8 +207,6 @@ char *homedir(void)
 }
 
 
-
-
 void config_skip_whitespace(struct configfile *f)
 {
      char c;
@@ -285,6 +304,12 @@ void config_get_token(struct configfile *f)
 	  f->cur_token = T_DATA;
 	  while( (c = f->rcfile[f->p]) && c != '<' && c != '>'
 		 && i < MAX_DATA_LEN) {
+	       /* escape next char */
+	       if(c == '\\') {
+		    if(f->rcfile[++(f->p)]) {
+			 c = f->rcfile[f->p];
+		    }
+	       }
 	       f->cur_string[i++] = c;
 	       if(c == '\n')
 		    f->line++;
@@ -421,14 +446,27 @@ void process_rcfile(char *rcfile)
 			      strncpy(f->cur_template->name, f->cur_string, 128);
 			      break;
 			 case T_OBJECTCLASS:
-			      f->cur_template->objectclasses = g_list_append(
-				   f->cur_template->objectclasses,
-				   strdup(f->cur_string));
+			      f->cur_template->objectclasses =
+				   g_list_append(f->cur_template->objectclasses, strdup(f->cur_string));
+			      break;
+			 }
+		    }
+		    else if(f->sp > 2 && f->stack[f->sp - 2] == T_FILTER) {
+			 switch(f->stack[f->sp - 1]) {
+			 case T_NAME:
+			      strncpy(f->cur_filter->name, f->cur_string, 127);
+			      break;
+			 case T_LDAPFILTER:
+			      strncpy(f->cur_filter->ldapfilter, f->cur_string, 1023);
+			      break;
+			 case T_SERVERNAME:
+			      strncpy(f->cur_filter->servername, f->cur_string, MAX_SERVERNAME_LEN - 1);
+			      break;
+			 case T_BASEDN:
+			      strncpy(f->cur_filter->basedn, f->cur_string, MAX_DN_LEN - 1);
 			      break;
 			 }
-
 		    }
-
 	       }
 
 
@@ -449,15 +487,21 @@ void process_rcfile(char *rcfile)
 		    else if(f->cur_token == T_TEMPLATE) {
 			 f->cur_template = new_template();
 		    }
+		    else if(f->cur_token == T_FILTER) {
+			 f->cur_filter = new_filter();
+		    }
 
 		    break;
 	       case CLOSE:
+		    if(f->cur_token == T_FILTER) {
+			 config.filters = g_list_append(config.filters, f->cur_filter);
+		    }
 		    if(f->cur_token_flags & NEEDS_CLOSE) {
 			 if(f->stack[(f->sp) - 1] == f->cur_token) {
 			      f->sp--;
 			 }
 			 else {
-			      sprintf(errstr, "line %d: can't close %s here",
+			      sprintf(errstr, "line %d: can't close %s here\n",
 				      f->line, f->cur_string);
 			      error_push(f->err_context, errstr);
 			 }
@@ -590,9 +634,24 @@ void config_write_int(struct writeconfig *wc, int value, char *entity)
 
 void config_write_string(struct writeconfig *wc, char *value, char *entity)
 {
-     char outstr[128];
+     int i, p;
+     char outstr[1024];
 
-     sprintf(outstr, "<%s>%s</%s>\n", entity, value, entity);
+     sprintf(outstr, "<%s>", entity);
+
+     /* quick-and-dirty escape < and > */
+     for(i = 0; value[i]; i++) {
+	  if(value[i] == '<' || value[i] == '>') {
+	       strcat(outstr, "\\");
+	  }
+	  p = strlen(outstr);
+	  outstr[p++] = value[i];
+	  outstr[p] = '\0';
+     }
+
+     strcat(outstr, "</");
+     strcat(outstr, entity);
+     strcat(outstr, ">\n");
      config_write(wc, outstr);
 
 }
@@ -609,9 +668,10 @@ void config_write_string_ne(struct writeconfig *wc, char *value, char *entity)
 
 void save_config(void)
 {
-     GList *templatelist, *oclist;
+     GList *templatelist, *oclist, *filterlist;
      struct ldapserver *server;
      struct gq_template *template;
+     struct gq_filter *filter;
      struct writeconfig *wc;
      int write_context;
      char *rcpath, errstr[256];
@@ -703,6 +763,26 @@ void save_config(void)
 	  templatelist = templatelist->next;
      }
 
+     /* filters */
+     filterlist = config.filters;
+     while(filterlist) {
+	  filter = (struct gq_filter *) filterlist->data;
+	  config_write(wc, "<filter>\n");
+	  wc->indent++;
+
+	  config_write_string(wc, filter->name, "name");
+	  config_write_string(wc, filter->ldapfilter, "ldapfilter");
+	  if(filter->servername[0])
+	       config_write_string(wc, filter->servername, "servername");
+	  if(filter->basedn[0])
+	       config_write_string(wc, filter->basedn, "basedn");
+
+	  wc->indent--;
+	  config_write(wc, "</filter>\n\n");
+
+	  filterlist = filterlist->next;
+     }
+
      wc->indent--;
      config_write(wc, "</gq-config>\n");
 
@@ -742,6 +822,7 @@ void init_config(void)
 
      config.ldapservers = NULL;
      config.templates = NULL;
+     config.filters = NULL;
      config.confirm_mod = 0;
      config.search_argument = SEARCHARG_BEGINS_WITH;
      config.showdn = DEFAULT_SHOWDN;
