@@ -21,8 +21,13 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <glib.h>
 
 void encode_password(GHashTable *hash);
 void encode_password_crypt(struct formfill *form);
 void encode_password_md5(struct formfill *form);
 void encode_password_sha1(struct formfill *form);
+const gchar *decode_string(gchar *native_string, const gchar *ldap_string);
+const gchar *encode_string(gchar *ldap_string, const gchar *native_string);
+gchar *decoded_string(const gchar *string);
+gchar *encoded_string(const gchar *string);
