@@ -24,10 +24,13 @@
 
 #include <gtk/gtk.h>
 
+#include "gq.h"
 #include "mainwin.h"
 #include "configfile.h"
 
 
+GdkFont *big_fixed;
+
 int main(int argc, char *argv[])
 {
 
@@ -36,6 +39,7 @@ int main(int argc, char *argv[])
 
      init_config();
 
+     setup_fonts();
      create_mainwin();
 
      gtk_main ();
@@ -44,3 +48,10 @@ int main(int argc, char *argv[])
 }
 
 
+void setup_fonts(void)
+{
+
+     big_fixed = gdk_font_load(FONT_BIG_FIXED);
+
+}
+
