@@ -221,7 +221,7 @@ void build_inputform(GHashTable *hash)
      struct formfill *form, *focusform;
      void *activatefunc;
      int row, row_arrow, vcnt, i, temp, max_width;
-     char *crypt_type, *dn;
+     char *crypt_type, *dn,  value[MAX_DN_LEN];
 
      gdk_color_alloc(gdk_colormap_get_system(), &red);
      not_in_schema = gtk_style_copy(gtk_widget_get_default_style());
@@ -252,7 +252,7 @@ void build_inputform(GHashTable *hash)
 				    GTK_SIGNAL_FUNC(activatefunc), (gpointer) hash);
      dn = g_hash_table_lookup(hash, "dn");
      if(dn)
-	  gtk_entry_set_text(GTK_ENTRY(inputbox), dn);
+	  gtk_entry_set_text(GTK_ENTRY(inputbox), decode_string(value, dn));
      /* set focus on DN for now, will get overridden by focusform in hash if present */
      gtk_editable_set_position(GTK_EDITABLE(inputbox), 0);
      gtk_widget_grab_focus(inputbox);
@@ -289,7 +289,8 @@ void build_inputform(GHashTable *hash)
 			 gtk_signal_connect_object(GTK_OBJECT(inputbox), "activate",
 						   GTK_SIGNAL_FUNC(activatefunc), (gpointer) hash);
 		    if(values) {
-			 gtk_entry_set_text(GTK_ENTRY(inputbox), values->data);
+		         gtk_entry_set_text(GTK_ENTRY(inputbox), 
+					    decode_string(value, values->data));
 			 values = values->next;
 		    }
 		    break;
@@ -352,7 +353,8 @@ void build_inputform(GHashTable *hash)
 				     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
 
 		    if(values) {
-			 gtk_entry_set_text(GTK_ENTRY(inputbox), values->data);
+			 gtk_entry_set_text(GTK_ENTRY(inputbox), 
+					    decode_string(value, values->data));
 			 values = values->next;
 		    }
 		    break;
@@ -483,6 +485,7 @@ void update_formlist(GHashTable *hash)
      struct formfill *form;
      int displaytype;
      char *content, *crypt_type, *old_dn;
+     char *content_enc;
 
      formlist = (GList *) g_hash_table_lookup(hash, "formlist");
      if(formlist == NULL)
@@ -538,19 +541,28 @@ void update_formlist(GHashTable *hash)
 		    }
 		    break;
 	       default:
-
+                    /*
+                     * Something's very wrong!  This should complain
+                     * and exit.  (syl)
+                     */
+                    break;
 	       }
 
 	       /* don't bother adding in empty fields */
-	       if(content && strlen(content))
-		    form->values = g_list_append(form->values, content);
+	       if(content && strlen(content)) {
+		    content_enc = encoded_string(content);
+		    g_free(content);
+		    form->values = g_list_append(form->values, content_enc);
+	       }
 	  }
 	  else if(displaytype == DISPLAYTYPE_DN) {
 	       content = gtk_editable_get_chars(GTK_EDITABLE(child->widget), 0, -1);
+	       content_enc = encoded_string(content);
+	       g_free(content);
 	       old_dn = g_hash_table_lookup(hash, "dn");
 	       if(old_dn)
 		    g_free(old_dn);
-	       g_hash_table_insert(hash, "dn", content);
+	       g_hash_table_insert(hash, "dn", content_enc);
 	  }
 
 	  children = g_list_previous(children);
@@ -771,7 +783,11 @@ int change_rdn(GHashTable *hash, int context)
 	  g_string_sprintfa(message, "modifying RDN to %s", rdn[0]);
 	  statusbar_msg(message->str);
 
+#if defined(HAVE_LDAP_RENAME)
+	  res = ldap_rename_s(ld, olddn, rdn[0], NULL, 1, NULL, NULL);
+#else
 	  res = ldap_modrdn2_s(ld, olddn, rdn[0], 1);
+#endif
 	  if(res != LDAP_SUCCESS) {
 	       g_string_sprintf(message, "ldap_modrdn2_s: %s", ldap_err2string(res));
 	       error_push(context, message->str);
