@@ -38,13 +38,14 @@
 #include "prefs.h"
 #include "util.h"
 #include "template.h"
+#include "filter.h"
 #include "COPYING.h"
 #include "debug.h"
 
 #include "../icons/logo.xpm"
 
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
-    #include "schemabrowse.h"
+#    include "schemabrowse.h"
 #endif
 
 GtkWidget *mainwin = NULL;
@@ -92,7 +93,7 @@ void gq_exit(GtkWidget *widget, gpointer *data)
 
 void cleanup(void)
 {
-     GList *templatelist, *oclist;
+     GList *templatelist, *oclist, *filterlist;
      struct ldapserver *server, *nextserver;
      struct gq_template *tmpl;
 
@@ -123,6 +124,14 @@ void cleanup(void)
      }
      g_list_free(config.templates);
 
+     /* free filters */
+     filterlist = config.filters;
+     while(filterlist) {
+	  FREE(filterlist->data, "struct gq_filter");
+	  filterlist = filterlist->next;
+     }
+     g_list_free(config.filters);
+
 #ifdef DEBUG
      report_num_mallocs();
 #endif
@@ -136,7 +145,10 @@ void create_mainwin(void)
      GtkWidget *File, *menuFile, *New, *Close, *Quit;
      GtkWidget *Search, *Browse, *Schema;
      GtkWidget *menuHelp, *Help, *License, *About;
+     GtkWidget *Filters, *menuFilters;
      GtkAccelGroup *accel_group;
+     GList *filterlist;
+     struct gq_filter *filter;
 
      mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
      gtk_window_set_default_size(GTK_WINDOW(mainwin), 770, 478);
@@ -168,7 +180,7 @@ void create_mainwin(void)
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(File), menuFile);
 
      /* File | New */
-     New = gtk_menu_item_new_with_label("New");
+     New = gtk_menu_item_new_with_label("New tab");
      gtk_widget_show(New);
      gtk_container_add(GTK_CONTAINER(menuFile), New);
      submenu = gtk_menu_new();
@@ -241,6 +253,66 @@ void create_mainwin(void)
 			       GTK_OBJECT(mainwin));
 
 
+     /* Filters menu */
+     Filters = gtk_menu_item_new_with_label("Filters");
+     gtk_widget_show(Filters);
+     gtk_container_add(GTK_CONTAINER(menubar), Filters);
+
+     menuFilters = gtk_menu_new();
+     gtk_menu_item_set_submenu(GTK_MENU_ITEM(Filters), menuFilters);
+     gtk_object_set_data(GTK_OBJECT(mainwin), "filtermenu", menuFilters);
+
+     /* Filters | New */
+     New = gtk_menu_item_new_with_label("New filter");
+     gtk_widget_show(New);
+     gtk_container_add(GTK_CONTAINER(menuFilters), New);
+     submenu = gtk_menu_new();
+     gtk_menu_item_set_submenu(GTK_MENU_ITEM(New), submenu);
+
+     /* Filters | New | From Search tab */
+     menuitem = gtk_menu_item_new_with_label("From Search tab");
+     gtk_widget_show(menuitem);
+     gtk_menu_append(GTK_MENU(submenu), menuitem);
+     gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
+			       GTK_SIGNAL_FUNC(add_filter),
+			       NULL);
+
+     /* Filters | New | Filter Editor */
+     menuitem = gtk_menu_item_new_with_label("Filter editor");
+     gtk_widget_show(menuitem);
+     gtk_menu_append(GTK_MENU(submenu), menuitem);
+     gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
+			       GTK_SIGNAL_FUNC(add_new_filter_callback),
+			       NULL);
+
+     /* Filters | Edit Filters */
+     menuitem = gtk_menu_item_new_with_label("Edit Filters");
+     gtk_widget_show(menuitem);
+     gtk_container_add(GTK_CONTAINER(menuFilters), menuitem);
+     gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
+			       GTK_SIGNAL_FUNC(show_filters),
+			       NULL);
+
+     /* Filters separator */
+     menuitem = gtk_menu_item_new();
+     gtk_widget_show(menuitem);
+     gtk_container_add(GTK_CONTAINER(menuFilters), menuitem);
+
+     /* Filters | list of filters */
+     filterlist = config.filters;
+     while(filterlist) {
+	  filter = (struct gq_filter *) filterlist->data;
+	  menuitem = gtk_menu_item_new_with_label(filter->name);
+	  gtk_object_set_data(GTK_OBJECT(menuitem), "filter", filter);
+	  gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
+				    GTK_SIGNAL_FUNC(filter_selected), (GtkObject *) filter);
+	  gtk_container_add(GTK_CONTAINER(menuFilters), menuitem);
+	  gtk_widget_show(menuitem);
+
+	  filterlist = filterlist->next;
+     }
+
+
      /* Help menu */
      Help = gtk_menu_item_new_with_label("Help");
      gtk_widget_show(Help);
