@@ -40,6 +40,7 @@
 #include "input.h"
 #include "template.h"
 #include "tinput.h"
+#include "filter.h"
 
 
 extern struct gq_config config;
@@ -363,7 +364,7 @@ void query(GHashTable *hash)
      struct resultset *set, *cur_resultset;
      struct attrs *attrlist;
      gchar *cur_servername, *cur_searchbase, *querystring, *cl[MAX_NUM_ATTRIBUTES];
-     char tolist[MAX_NUM_ATTRIBUTES][128], message[1152], filter[1024];
+     char tolist[MAX_NUM_ATTRIBUTES][128], message[1152], filter[MAX_LDAPFILTER_LEN];
      char *attr, *dn, **vals;
      int searchtype, msg, rc, i, row;
      int cur_col, oc_col, columns_done[MAX_NUM_ATTRIBUTES];
@@ -416,22 +417,22 @@ void query(GHashTable *hash)
      case ST_SEARCH:
 	  switch(config.search_argument) {
 	  case SEARCHARG_BEGINS_WITH:
-	       sprintf(filter, "(%s=%s*)", server->searchattr, querystring);
+	       snprintf(filter, MAX_LDAPFILTER_LEN - 1, "(%s=%s*)", server->searchattr, querystring);
 	       break;
 	  case SEARCHARG_ENDS_WITH:
-	       sprintf(filter, "(%s=*%s)", server->searchattr, querystring);
+	       snprintf(filter, MAX_LDAPFILTER_LEN - 1, "(%s=*%s)", server->searchattr, querystring);
 	       break;
 	  case SEARCHARG_CONTAINS:
-	       sprintf(filter, "(%s=*%s*)", server->searchattr, querystring);
+	       snprintf(filter, MAX_LDAPFILTER_LEN - 1, "(%s=*%s*)", server->searchattr, querystring);
 	       break;
 	  case SEARCHARG_EQUALS:
-	       sprintf(filter, "(%s=%s)", server->searchattr, querystring);
+	       snprintf(filter, MAX_LDAPFILTER_LEN - 1, "(%s=%s)", server->searchattr, querystring);
 	       break;
 
 	  };
 	  break;
      case ST_FILTER:
-	  strncpy(filter, querystring, 1024);
+	  strncpy(filter, querystring, MAX_LDAPFILTER_LEN - 1);
 	  break;
      };
 
@@ -449,8 +450,8 @@ void query(GHashTable *hash)
 	  msg = ldap_search(ld, cur_searchbase, LDAP_SCOPE_SUBTREE,
 			    filter, NULL, 0);
 
-     if(msg == -1) {
-	  statusbar_msg(ldap_err2string(msg));
+     if((config.sort_search && msg != LDAP_SUCCESS) || msg == -1) {
+	  statusbar_msg(ldap_err2string(msg == -1 ? LDAP_OTHER : msg));
 	  g_hash_table_remove(hash, "search lock");
 	  return;
      }
