@@ -36,7 +36,7 @@
 #include <lber.h>
 #include <ldap.h>
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
-    #include <ldap_schema.h>
+#    include <ldap_schema.h>
 #endif
 
 
@@ -130,15 +130,15 @@ LDAP *open_connection(struct ldapserver *server)
 
 	       msg = LDAP_SUCCESS;
 	       if(server->bindtype == BINDTYPE_KERBEROS) {
-		    #ifdef HAVE_KERBEROS
+#		    ifdef HAVE_KERBEROS
 		    msg = ldap_bind_s(ld, binddn, bindpw, LDAP_AUTH_KRBV4);
-		    #else
+#		    else
 		    error_push(open_context, "GQ was compiled without Kerberos support.\n"
 			       "Run 'configure --help' for more information\n");
 		    statusbar_msg("");
 		    ldap_unbind(ld);
 		    ld = NULL;
-		    #endif
+#		    endif
 	       }
 	       else
 		    msg = ldap_simple_bind_s(ld, binddn, bindpw);
@@ -487,6 +487,7 @@ int is_leaf_entry(struct ldapserver *server, char *dn)
 	  if( (ldap_result(ld, msg, 0, NULL, &res) != LDAP_RES_SEARCH_ENTRY))
 	       is_leaf = 1;
 	  ldap_msgfree(res);
+	  ldap_abandon(ld, msg);
      }
 
      close_connection(server, FALSE);
@@ -589,6 +590,16 @@ void warning_popup(GList *messages)
 }
 
 
+void single_warning_popup(char *message)
+{
+     GList *msglist;
+
+     msglist = g_list_append(NULL, message);
+     warning_popup(msglist);
+
+}
+
+
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
 GList *find_at_by_mr_oid(struct ldapserver *server, char *oid)
 {
